port module Counter exposing (main)

import AMR
import Browser
import Html exposing (button, div, text)
import Html.Events exposing (onClick)
import Json.Decode as Decode
import Json.Encode as Encode


type Msg
    = Increment
    | Decrement
    | DoNothing


port toCache : String -> Cmd msg


amrConfig =
    { encodeMsg =
        \msg ->
            case msg of
                Increment ->
                    Encode.string "Increment"

                Decrement ->
                    Encode.string "Decrement"

                DoNothing ->
                    Encode.string "DoNothing"
    , msgDecoder =
        Decode.andThen
            (\text ->
                case text of
                    "Increment" ->
                        Decode.succeed Increment

                    "Decrement" ->
                        Decode.succeed Decrement

                    "DoNothing" ->
                        Decode.succeed DoNothing

                    _ ->
                        Decode.fail ("'" ++ text ++ "' is not a valid message!")
            )
            Decode.string
    , fromCache = identity
    , toCache = toCache
    }


main : AMR.Program (Maybe String) Int Msg
main =
    AMR.application amrConfig
        -- Browser.application
        { onUrlRequest = always DoNothing
        , onUrlChange = always DoNothing
        , subscriptions = \_ -> Sub.none
        , init = \_ _ _ -> ( 0, Cmd.none )
        , update =
            \msg count ->
                case msg of
                    DoNothing ->
                        ( count, Cmd.none )

                    Increment ->
                        ( count + 1, Cmd.none )

                    Decrement ->
                        ( count - 1, Cmd.none )
        , view =
            \count ->
                { title = "Counter"
                , body =
                    [ button [ onClick Increment ] [ text "+" ]
                    , div [] [ text (String.fromInt count) ]
                    , button [ onClick Decrement ] [ text "-" ]
                    ]
                }
        }
